# -*- coding: utf-8 -*-
#
# Copyright (c) ZeroC, Inc. All rights reserved.
#
#
# Ice version 3.7.9
#
# <auto-generated>
#
# Generated from file `ConnectionF.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy

# Start of module Ice
_M_Ice = Ice.openModule('Ice')
__name__ = 'Ice'

if 'ConnectionInfo' not in _M_Ice.__dict__:
    _M_Ice._t_ConnectionInfo = IcePy.declareValue('::Ice::ConnectionInfo')

if 'WSConnectionInfo' not in _M_Ice.__dict__:
    _M_Ice._t_WSConnectionInfo = IcePy.declareValue('::Ice::WSConnectionInfo')

if 'Connection' not in _M_Ice.__dict__:
    _M_Ice._t_Connection = IcePy.declareValue('::Ice::Connection')

# End of module Ice
