# -*- coding: utf-8 -*-
#
# Copyright (c) ZeroC, Inc. All rights reserved.
#
#
# Ice version 3.7.9
#
# <auto-generated>
#
# Generated from file `BuiltinSequences.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy

# Start of module Ice
_M_Ice = Ice.openModule('Ice')
__name__ = 'Ice'

if '_t_BoolSeq' not in _M_Ice.__dict__:
    _M_Ice._t_BoolSeq = IcePy.defineSequence('::Ice::BoolSeq', (), IcePy._t_bool)

if '_t_ByteSeq' not in _M_Ice.__dict__:
    _M_Ice._t_ByteSeq = IcePy.defineSequence('::Ice::ByteSeq', (), IcePy._t_byte)

if '_t_ShortSeq' not in _M_Ice.__dict__:
    _M_Ice._t_ShortSeq = IcePy.defineSequence('::Ice::ShortSeq', (), IcePy._t_short)

if '_t_IntSeq' not in _M_Ice.__dict__:
    _M_Ice._t_IntSeq = IcePy.defineSequence('::Ice::IntSeq', (), IcePy._t_int)

if '_t_LongSeq' not in _M_Ice.__dict__:
    _M_Ice._t_LongSeq = IcePy.defineSequence('::Ice::LongSeq', (), IcePy._t_long)

if '_t_FloatSeq' not in _M_Ice.__dict__:
    _M_Ice._t_FloatSeq = IcePy.defineSequence('::Ice::FloatSeq', (), IcePy._t_float)

if '_t_DoubleSeq' not in _M_Ice.__dict__:
    _M_Ice._t_DoubleSeq = IcePy.defineSequence('::Ice::DoubleSeq', (), IcePy._t_double)

if '_t_StringSeq' not in _M_Ice.__dict__:
    _M_Ice._t_StringSeq = IcePy.defineSequence('::Ice::StringSeq', (), IcePy._t_string)

if '_t_ObjectSeq' not in _M_Ice.__dict__:
    _M_Ice._t_ObjectSeq = IcePy.defineSequence('::Ice::ObjectSeq', (), IcePy._t_Value)

if '_t_ObjectProxySeq' not in _M_Ice.__dict__:
    _M_Ice._t_ObjectProxySeq = IcePy.defineSequence('::Ice::ObjectProxySeq', (), IcePy._t_ObjectPrx)

# End of module Ice

Ice.sliceChecksums["::Ice::BoolSeq"] = "321b1d4186eaf796937e82eed711fe7f"
Ice.sliceChecksums["::Ice::ByteSeq"] = "eae189e6d7b57b6f9628e78293d6d7"
Ice.sliceChecksums["::Ice::DoubleSeq"] = "7fb8b78b3ab4b2e358d27dc4d6f1e330"
Ice.sliceChecksums["::Ice::FloatSeq"] = "eb848491fad1518427fd58a7330958e"
Ice.sliceChecksums["::Ice::IntSeq"] = "826eab2e95b89b0476198b6a36991dc"
Ice.sliceChecksums["::Ice::LongSeq"] = "325a51044cdee10b14ecc3c1672a86"
Ice.sliceChecksums["::Ice::ObjectProxySeq"] = "45ccc241c2bbfebfbd031e733a5832"
Ice.sliceChecksums["::Ice::ObjectSeq"] = "a1e80bc6b87e9687455a4faa4c2f295"
Ice.sliceChecksums["::Ice::ShortSeq"] = "32aadcc785b88fc7198ae3d262ff3acd"
Ice.sliceChecksums["::Ice::StringSeq"] = "7986f19514e3b04fbe4af53c7446e563"
