# -*- coding: utf-8 -*-
#
# Copyright (c) ZeroC, Inc. All rights reserved.
#
#
# Ice version 3.7.9
#
# <auto-generated>
#
# Generated from file `PluginF.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy

# Start of module Ice
_M_Ice = Ice.openModule('Ice')
__name__ = 'Ice'

if 'Plugin' not in _M_Ice.__dict__:
    _M_Ice._t_Plugin = IcePy.declareValue('::Ice::Plugin')

if 'PluginManager' not in _M_Ice.__dict__:
    _M_Ice._t_PluginManager = IcePy.declareValue('::Ice::PluginManager')

# End of module Ice
